@@ -6,7 +6,7 @@
  *  these oopses for good, so we run those stupid tests... */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2015 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,11 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "exams.h"
+#include "ginac.h"
+using namespace GiNaC;
+
+#include <iostream>
+using namespace std;
 
 // The very first pair of historic problems had its roots in power.cpp and was
 // finally resolved on April 27th 1999. (Fixing the first on April 23rd
@@ -283,7 +287,7 @@ static unsigned exam_paranoia11()
 
 	ex e = ((-5-2*x)-((2-5*x)/(-2+x))*(3+2*x))/(5-4*x);
 	ex f = e.normal();
-	ex d = (4+10*x+8*pow(x,2))/(x-2)/(5-4*x);
+	ex d = normal((4+10*x+8*pow(x,2))/(x-2)/(5-4*x));
 
 	if (!(f - d).expand().is_zero()) {
 		clog << "normal(" << e << ") returns " << f << " instead of " << d << endl;
@@ -437,12 +441,138 @@ static unsigned exam_paranoia16()
 	return result;
 }
 
+// Bug in reposition_dummy_indices() could result in correct expression
+// turned into one with inconsistent indices. Fixed on Aug 29, 2006
+static unsigned exam_paranoia17()
+{
+	varidx mu1(symbol("mu1"), 4);
+	varidx mu2(symbol("mu2"), 4);
+	varidx mu3(symbol("mu3"), 4);
+	varidx mu4(symbol("mu4"), 4);
+	varidx mu5(symbol("mu5"), 4);
+	varidx mu6(symbol("mu6"), 4);
+
+	exvector ev2;
+	ev2.push_back(mu3.toggle_variance());
+	ev2.push_back(mu6);
+	ev2.push_back(mu5.toggle_variance());
+	ev2.push_back(mu6.toggle_variance());
+	ev2.push_back(mu5);
+	ev2.push_back(mu3); 
+	// notice: all indices are contracted ...
+
+	ex test_cycl = indexed(symbol("A"), sy_cycl(), ev2);
+	test_cycl = test_cycl.simplify_indexed();
+	// ... so there should be zero free indices in the end.
+	return test_cycl.get_free_indices().size();
+}
+
+// Bug in add::eval() could result in numeric terms not being collected into
+// the overall coefficient. Fixed on Sep 22, 2010
+static unsigned exam_paranoia18()
+{
+	ex sqrt2 = sqrt(ex(2));
+	ex e = 1+2*(sqrt2+1)*(sqrt2-1);
+	if ( e.real_part() != 3 ) {
+		clog << "real_part(1+2*(sqrt(2)+1)*(sqrt(2)-1)) failed to evaluate to 3\n";
+		return 1;
+	}
+	return 0;
+}
+
+// Bug in mul::conjugate when factors are evaluated at branch cuts, reported as
+// Sage bug #10964.
+static unsigned exam_paranoia19()
+{
+	symbol a("a");
+	ex e = conjugate(a*sqrt(ex(-2))*sqrt(ex(-3)));
+	ex c = a*conjugate(sqrt(ex(-2)))*conjugate(sqrt(ex(-3)));
+	if (!subs(e-c, a==42).is_zero()) {
+		clog << "subs(a*conjugate(sqrt(-2))*conjugate(sqrt(-3))-conjugate(a*sqrt(-2)*sqrt(-3)),a==42) failed to evaluate to 0\n";
+		return 1;
+	}
+	return 0;
+}
+
+// Bugs in is_polynomial (fixed 2011-05-20 and 2014-07-26).
+static unsigned exam_paranoia20()
+{
+	unsigned result = 0;
+	symbol x("x"), y("y");
+	ex e1 = sqrt(x*x+1)*sqrt(x+1);
+	if (e1.is_polynomial(x)) {
+		clog << "sqrt(x*x+1)*sqrt(x+1) is wrongly reported to be a polynomial in x\n";
+		++result;
+	}
+	ex e2 = sqrt(Pi)*x;
+	if (!e2.is_polynomial(x)) {
+		clog << "sqrt(Pi)*x is wrongly reported to be no polynomial in x\n";
+		++result;
+	}
+	ex e3 = sqrt(x);
+	if (!e3.is_polynomial(y)) {
+		clog << "sqrt(x) is wrongly reported to be no polynomial in y\n";
+		++result;
+	}
+	ex e4 = (1+y)/(2+x);
+	if (e4.is_polynomial(x)) {
+		clog << "(1+y)/(2+x) is wrongly reported to be a polynomial in x\n";
+		++result;
+	}
+	return result;
+}
+
+static unsigned is_polynomial_false_positive()
+{
+	unsigned result = 0;
+	symbol x("x"), n("n");
+	exvector nonpoly_exprs;
+	nonpoly_exprs.push_back(1/(1-x));
+	nonpoly_exprs.push_back(1/(x+1));
+	nonpoly_exprs.push_back(-1/(x-1));
+	nonpoly_exprs.push_back(1/(1-x*x));
+	nonpoly_exprs.push_back(1/(1-pow(x,n)));
+	nonpoly_exprs.push_back(x-1/(x-1));
+	for (exvector::const_iterator ep = nonpoly_exprs.begin();
+	     ep != nonpoly_exprs.end(); ++ep) {
+		if (ep->is_polynomial(x)) {
+			clog << "(" << *ep << ").is_polynomial(" << x << ") "
+			        "erroneously returned true" << endl;
+			++result;
+		}
+	}
+	return result;
+}
+
+// Bug in power::expand reported by Isuru Fernando (fixed 2015-05-07).
+static unsigned exam_paranoia21()
+{
+	symbol x("x");
+	ex e = pow(x + sqrt(ex(2))*x, 2).expand();
+	if (e.nops() != 2) {
+		clog << "(x+sqrt(2)*x)^2 was wrongly expanded to " << e << "\n";
+		return 1;
+	}
+	return 0;
+}
+
+// Bug in power::expand (fixed 2015-07-18).
+static unsigned exam_paranoia22()
+{
+	symbol x("x"), y("y");
+	ex e = pow(sqrt(1+x)+y*sqrt(1+x), 2).expand();
+	if (e.nops() != 6) {
+		clog << "(sqrt(1+x)+y*sqrt(1+x))^2 was wrongly expanded to " << e << "\n";
+		return 1;
+	}
+	return 0;
+}
+
 unsigned exam_paranoia()
 {
 	unsigned result = 0;
 	
 	cout << "examining several historic failures just out of paranoia" << flush;
-	clog << "----------several historic failures:" << endl;
 	
 	result += exam_paranoia1();  cout << '.' << flush;
 	result += exam_paranoia2();  cout << '.' << flush;
@@ -460,13 +590,18 @@ unsigned exam_paranoia()
 	result += exam_paranoia14();  cout << '.' << flush;
 	result += exam_paranoia15();  cout << '.' << flush;
 	result += exam_paranoia16();  cout << '.' << flush;
-	
-	if (!result) {
-		cout << " passed " << endl;
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed " << endl;
-	}
+	result += exam_paranoia17();  cout << '.' << flush;
+	result += exam_paranoia18();  cout << '.' << flush;
+	result += exam_paranoia19();  cout << '.' << flush;
+	result += exam_paranoia20();  cout << '.' << flush;
+	result += is_polynomial_false_positive(); cout << '.' << flush;
+	result += exam_paranoia21();  cout << '.' << flush;
+	result += exam_paranoia22();  cout << '.' << flush;
 	
 	return result;
 }
+
+int main(int argc, char** argv)
+{
+	return exam_paranoia();
+}
